package com.ihaoin.hooloo.device.network;

import android.os.Handler;
import android.os.Message;
import android.util.Log;

import java.io.IOException;
import java.net.NetworkInterface;
import java.util.Collections;
import java.util.List;

import okhttp3.Call;
import okhttp3.Callback;
import okhttp3.MediaType;
import okhttp3.OkHttpClient;
import okhttp3.Request;
import okhttp3.RequestBody;
import okhttp3.Response;

public class HttpUtil {
    private static final String TAG = "Network";

    public static void post(String url, String body, Handler handler) {
        OkHttpClient client = new OkHttpClient();
        RequestBody requestBody = RequestBody.create(MediaType.parse("application/json"), body);
        Request request = new Request.Builder().url(url).post(requestBody).build();
        client.newCall(request).enqueue(new Callback() {
            @Override
            public void onFailure(Call call, IOException e) {
                HttpUtil.onFailure(handler, e);
            }

            @Override
            public void onResponse(Call call, Response response) throws IOException {
                HttpUtil.onResponse(handler, response);
            }
        });
    }

    private static void onFailure(Handler handler, Exception e) {
        if (handler == null) {
            String msgText = String.format("返回状态:%s, 返回内容:%s", 0, e.getMessage());
            Log.d(TAG, msgText);
            return;
        }
        try {
            Thread.sleep(3000);
        } catch (InterruptedException e3) {
            e3.printStackTrace();
        }
        Message msg = handler.obtainMessage(0, e);
        handler.sendMessage(msg);
    }

    private static void onResponse(Handler handler, Response response) throws IOException {
        if (handler == null) {
            String msgText = String.format("返回状态:%s, 返回内容:%s", 1, response.body() == null ? "" : response.body().string());
            Log.d(TAG, msgText);
            return;
        }
        try {
            Thread.sleep(3000);
        } catch (InterruptedException e) {
            e.printStackTrace();
        }
        Message msg = handler.obtainMessage(1, response.body() == null ? "" : response.body().string());
        handler.sendMessage(msg);
    }

    public static void getMainDatas(String body, Handler handler) {
        post(BaseParams.GET_MAIN_DATA, body, handler);
    }

    public static void getOrders(String body, Handler handler) {
        post(BaseParams.GET_ORDERS, body, handler);
    }

    public static void postConfirmOrder(String body, Handler handler) {
        post(BaseParams.POST_CONFIRM_ORDER, body, handler);
    }

    public static void postRegistrationId(String body, Handler handler) {
        post(BaseParams.POST_REGISTRATION_ID, body, handler);
    }

    public static void checkSku(String body, Handler handler) {
        post(BaseParams.GET_CHECK_SKU, body, handler);
    }

    public static String getMacAddress() {
        try {
            List<NetworkInterface> all = Collections.list(NetworkInterface.getNetworkInterfaces());
            for (NetworkInterface nif : all) {
                if (!nif.getName().equalsIgnoreCase("wlan0")) continue;
                byte[] macBytes = nif.getHardwareAddress();
                if (macBytes == null) {
                    return null;
                }
                StringBuilder res1 = new StringBuilder();
                for (byte b : macBytes) {
                    res1.append(String.format("%02X:", b));
                }
                if (res1.length() > 0) {
                    res1.deleteCharAt(res1.length() - 1);
                }
                return res1.toString();
            }
        } catch (Exception ex) {
            ex.printStackTrace();
        }
        return null;
    }
}
