package com.ihaoin.hooloo.device.view;

import android.app.Dialog;
import android.content.Context;
import android.content.DialogInterface;
import android.graphics.drawable.AnimationDrawable;
import android.os.Bundle;
import android.os.Handler;
import android.os.Message;
import android.util.Log;
import android.view.View;
import android.widget.TextView;
import android.widget.Toast;

import androidx.annotation.NonNull;

import com.alibaba.fastjson2.JSON;
import com.aliyun.alink.dm.api.DeviceInfo;
import com.aliyun.alink.linkkit.api.ILinkKitConnectListener;
import com.aliyun.alink.linkkit.api.IoTMqttClientConfig;
import com.aliyun.alink.linkkit.api.LinkKit;
import com.aliyun.alink.linkkit.api.LinkKitInitParams;
import com.aliyun.alink.linksdk.channel.core.persistent.mqtt.MqttConfigure;
import com.aliyun.alink.linksdk.tmp.device.payload.ValueWrapper;
import com.aliyun.alink.linksdk.tools.AError;
import com.ihaoin.hooloo.device.R;
import com.ihaoin.hooloo.device.component.NetworkHandler;
import com.ihaoin.hooloo.device.config.AppConfig;
import com.ihaoin.hooloo.device.data.TrolleyGoods;
import com.ihaoin.hooloo.device.data.po.ConfirmGoods;
import com.ihaoin.hooloo.device.data.po.ConfirmOrder;
import com.ihaoin.hooloo.device.data.vo.Goods;
import com.ihaoin.hooloo.device.data.vo.Sku;
import com.ihaoin.hooloo.device.network.HttpUtil;
import com.ihaoin.hooloo.device.util.RandomUtils;
import com.ihaoin.hooloo.device.util.SharedPreferencesUtils;
import com.ihaoin.hooloo.device.util.Utils;

import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

public class ConfirmOrderDialog extends Dialog {
    private List<ConfirmGoods> goods = new ArrayList<>();
    private UIHandler uiHandler;
    private ConfirmOrderHandler confirmOrderHandler;
    private Thread expireThread;
    private Thread returnThread;
    private final Integer EXPIRE_TIMES = 60;
    private final Integer RETURN_TIMES = 5;

    private View layoutContent;
    private View layoutConfirm;
    private View layoutSucceed;
    private TextView txtExpire;
    private TextView txtReturn;
    private View butnClose;
    private View loading;

    public ConfirmOrderDialog(@NonNull Context context) {
        super(context);
    }

    public void setGoods(List<TrolleyGoods> trolleyGoods) {
        this.goods = Utils.translate(trolleyGoods);
    }

    public void setGoods(Goods goods, Sku sku, Integer count) {
        this.goods = Utils.translate(goods, sku, count);
    }

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        this.setContentView(R.layout.view_confirm_order);
        initViews();

    }

    private void initViews() {
        layoutContent = findViewById(R.id.layout_content);
        layoutConfirm = findViewById(R.id.layout_confirm);
        layoutSucceed = findViewById(R.id.layout_succeed);
        txtExpire = findViewById(R.id.txt_expire);
        txtReturn = findViewById(R.id.txt_return);
        butnClose = findViewById(R.id.butn_close);
        loading = findViewById(R.id.loading);

        layoutConfirm.setVisibility(View.VISIBLE);
        layoutSucceed.setVisibility(View.GONE);

        butnClose.setOnClickListener(x -> dismiss());
        uiHandler = new UIHandler();

        setOnDismissListener(dismissListener);

        setCancelable(false);
        setCanceledOnTouchOutside(false);

        postConfirmOrder();
        connectMQTT();
    }

    private void connectMQTT() {
        /**
         * 设置设备认证信息
         */
        String clientId = AppConfig.MACHINE_CODE;
        String productKey = "";
        String deviceName = "";
        String deviceSecret = "";
        String deviceToken = "";

        DeviceInfo deviceInfo = new DeviceInfo();
        deviceInfo.productKey = productKey;// 产品类型
        deviceInfo.deviceName = deviceName;// 设备名称
        deviceInfo.deviceSecret = deviceSecret;// 设备密钥
        // 如果使用deviceToken和clientID连接物联网平台，那么设备的deviceSecret需要设置为null
        // 一型一密免白名单动态注册之后建联需要设置deviceToken、clientId这两个值，这两个值由deviceDynamicRegister接口返回
        MqttConfigure.deviceToken = deviceToken;
        MqttConfigure.clientId = clientId;
        /**
         * 设置设备当前的初始状态值，属性需要和物联网平台创建的物模型属性一致
         * 若此处为空，物模型属性的初始值则为空。
         * 调用物模型上报接口之后，物模型会有相关数据缓存。
         */
        Map<String, ValueWrapper> propertyValues = new HashMap<>();
        // 示例
        // propertyValues.put("LightSwitch", new ValueWrapper.BooleanValueWrapper(0));
        IoTMqttClientConfig clientConfig = new IoTMqttClientConfig(productKey, deviceName, deviceSecret);
        LinkKitInitParams params = new LinkKitInitParams();
        params.deviceInfo = deviceInfo;
        params.propertyValues = propertyValues;
        params.mqttClientConfig = clientConfig;
        /**
         * 设备初始化建联
         * onError 初始化建联失败，需要用户重试初始化。如因网络问题导致初始化失败。
         * onInitDone 初始化成功
         */
        LinkKit.getInstance().init(getContext(), params, new ILinkKitConnectListener() {
            @Override
            public void onError(AError error) {
                // 初始化失败 error包含初始化错误信息
            }

            @Override
            public void onInitDone(Object data) {
                // 初始化成功 data 作为预留参数
            }
        });

    }

    private void postConfirmOrder() {
        try {
            startLoading();

            confirmOrderHandler = new ConfirmOrderHandler(getContext());
            HttpUtil.postConfirmOrder(genJson(), confirmOrderHandler);
        } catch (Exception e) {
            e.printStackTrace();
            Toast.makeText(getContext(), R.string.error_get_qrcode, Toast.LENGTH_LONG).show();
            dismiss();
        }
    }

    private void startLoading() {
        layoutContent.setVisibility(View.INVISIBLE);
        loading.setVisibility(View.VISIBLE);

        AnimationDrawable animationDrawable = (AnimationDrawable) loading.getBackground();
        animationDrawable.start();
    }

    private void stopLoading() {
        layoutContent.setVisibility(View.VISIBLE);
        loading.setVisibility(View.INVISIBLE);

        AnimationDrawable animationDrawable = (AnimationDrawable) loading.getBackground();
        animationDrawable.stop();
    }

    private String genJson() {
        try {
            ConfirmOrder confirmOrder = new ConfirmOrder();
            confirmOrder.setMachineCode(AppConfig.MACHINE_CODE);
            confirmOrder.setScreenNo(SharedPreferencesUtils.getScreenNo(getContext()));
            confirmOrder.setSeqNo(genSeqNo());
            confirmOrder.setGoods(goods);
            return JSON.toJSONString(confirmOrder);
        } catch (Exception e) {
            e.printStackTrace();
        }
        return null;
    }

    private String genSeqNo() {
        return new Date().getTime() + "-" + RandomUtils.nextInt(null);
    }

    OnDismissListener dismissListener = new OnDismissListener() {
        @Override
        public void onDismiss(DialogInterface dialog) {
            Utils.sendClearGoodsBroadcast(getContext());
            if (expireThread != null) {
                expireThread.interrupt();
            }
            if (returnThread != null) {
                returnThread.interrupt();
            }
            if (uiHandler != null) {
                uiHandler.removeCallbacksAndMessages(null);
                uiHandler = null;
            }
            if (confirmOrderHandler != null) {
                confirmOrderHandler.removeCallbacksAndMessages(null);
                confirmOrderHandler = null;
            }
        }
    };

    private void startExpireThread() {
        expireThread = new Thread() {
            @Override
            public void run() {
                try {
                    for (int i = EXPIRE_TIMES; i >= 0; i--) {
                        if (uiHandler == null) {
                            return;
                        }
                        Message msg = new Message();
                        msg.what = 0;// 二维码失效倒计时
                        msg.obj = i;
                        uiHandler.sendMessage(msg);
                        Thread.sleep(1000);
                    }
                } catch (InterruptedException e) {
                    e.printStackTrace();
                }
            }
        };
        expireThread.start();
    }

    private void startReturnThread() {
        returnThread = new Thread() {
            @Override
            public void run() {
                try {
                    for (int i = RETURN_TIMES; i >= 0; i--) {
                        if (uiHandler == null) {
                            return;
                        }
                        Message msg = new Message();
                        msg.what = 1;// 扫码成功倒计时
                        msg.obj = i;
                        uiHandler.sendMessage(msg);
                        Thread.sleep(1000);
                    }
                } catch (InterruptedException e) {
                    e.printStackTrace();
                }
            }
        };
        returnThread.start();
    }

    class UIHandler extends Handler {
        @Override
        public void handleMessage(@NonNull Message msg) {
            switch (msg.what) {
                case 0:
                    Integer expireTimes = (Integer) msg.obj;
                    txtExpire.setText(getContext().getString(R.string.expire_times, expireTimes.toString()));
                    if (expireTimes <= 0) {
                        dismiss();
                    }
                    break;
                case 1:
                    Integer returnTimes = (Integer) msg.obj;
                    txtReturn.setText(getContext().getString(R.string.scan_succeed, returnTimes.toString()));
                    if (returnTimes <= 0) {
                        dismiss();
                    }
                    break;
            }
        }
    }

    class ConfirmOrderHandler extends NetworkHandler {
        public ConfirmOrderHandler(@NonNull Context context) {
            super(context);
        }

        @Override
        public void handleMessage(@NonNull Message msg) {
            stopLoading();
            if (msg.what == 0) {
                Toast.makeText(getContext(), R.string.error_get_qrcode, Toast.LENGTH_LONG).show();
                dismiss();
                return;
            }

            String s = (String) msg.obj;
            Log.d("HTTP", s);
            startExpireThread();
        }
    }
}
